<?php

namespace App\Http\Controllers;

use App\Models\Application;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Http;

class ApplicationController extends Controller
{
    public function index()
    {
        $apps = Application::paginate(10);
        return view('apps.index', compact('apps'));
    }

    public function create()
    {
        return view('apps.create');
    }
    public function store(Request $request)
{
    $request->validate([
        'nombre' => 'required|string|max:255',
        'descripcion' => 'nullable|string',
        'archivo_zip' => 'required|file|mimes:zip|max:100048', // Máximo ~100MB
    ]);

    try {
        // Subir el archivo ZIP
        $zipPath = $request->file('archivo_zip')->store('apps', 'public');

        // Guardar en la base de datos
        $app = Application::create([
            'nombre' => $request->nombre,
            'descripcion' => $request->descripcion,
            'archivo_zip' => $zipPath,
            'apk_path' => null, // Será definido al descomprimir
            'url' => null,      // Será definido al descomprimir
        ]);

        // Verificar si es una solicitud AJAX
        if ($request->expectsJson()) {
            return response()->json(['success' => true, 'message' => 'Aplicación subida correctamente.'], 200);
        }

        return redirect()->route('apps.index')->with('success', 'Aplicación subida correctamente. Ahora puedes descomprimir el archivo.');
    } catch (\Exception $e) {
        Log::error('Error al subir la aplicación: ' . $e->getMessage());

        if ($request->expectsJson()) {
            return response()->json(['error' => 'Error al subir la aplicación: ' . $e->getMessage()], 500);
        }

        return redirect()->back()->with('error', 'Error al subir la aplicación.');
    }
}
public function descomprimir($id)
{
    try {
        $modelo = Modelo::findOrFail($id);

        if (!$modelo->archivo_zip) {
            return redirect()->back()->with('error', 'El modelo no tiene un archivo ZIP asignado.');
        }

        // Ruta del archivo ZIP
        $zipPath = storage_path('app/public/' . $modelo->archivo_zip);

        // Nueva ruta donde se descomprimirá
        $extractPath = public_path('storage/modelos/' . $modelo->id);

        // Eliminar la carpeta anterior si existe
        if (is_dir($extractPath)) {
            $this->deleteDirectory($extractPath);
        }

        // Crear la carpeta si no existe
        if (!is_dir($extractPath)) {
            mkdir($extractPath, 0755, true);
        }

        // Descomprimir el archivo ZIP
        $zip = new \ZipArchive;
        if ($zip->open($zipPath) === TRUE) {
            $zip->extractTo($extractPath);
            $zip->close();

            // Guardar la ruta descomprimida y URL
            $modelo->carpeta_descomprimida = 'modelos/' . $modelo->id;
            $modelo->url = asset('storage/modelos/' . $modelo->id . '/index.html');
            $modelo->save();

            \Log::info('Modelo descomprimido correctamente.', ['modelo_id' => $modelo->id]);
            return redirect()->back()->with('success', 'Modelo descomprimido y URL generada correctamente.');
        } else {
            throw new \Exception('No se pudo abrir el archivo ZIP.');
        }
    } catch (\Exception $e) {
        \Log::error('Error al descomprimir el modelo.', ['exception' => $e->getMessage()]);
        return redirect()->back()->with('error', 'Error al descomprimir el modelo: ' . $e->getMessage());
    }
}

// Método auxiliar para eliminar directorio
private function deleteDirectory($directoryPath)
{
    $files = array_diff(scandir($directoryPath), ['.', '..']);
    foreach ($files as $file) {
        $path = $directoryPath . DIRECTORY_SEPARATOR . $file;
        is_dir($path) ? $this->deleteDirectory($path) : unlink($path);
    }
    rmdir($directoryPath);
}

    public function generateQr($id)
    {
        try {
            // Buscar la aplicación
            $app = Application::findOrFail($id);
    
            // Validar si el APK ha sido descomprimido
            if (!$app->apk_path) {
                return redirect()->back()->with('error', 'No se ha descomprimido el archivo APK para esta aplicación.');
            }
    
            // Generar la URL local del APK
            $app->url = 'http://192.168.1.200:8000/storage/' . $app->apk_path;
    
            // Generar el código QR con la URL del APK
            $qrCode = \SimpleSoftwareIO\QrCode\Facades\QrCode::format('svg')
                ->size(300)
                ->generate($app->url);
    
            // Retornar la vista con la aplicación y el QR generado
            return view('apps.qr', compact('app', 'qrCode'))
                ->with('success', 'Código QR generado correctamente.');
        } catch (\Exception $e) {
            Log::error('Error al generar QR.', ['exception' => $e->getMessage()]);
            return redirect()->back()->with('error', 'Error al generar QR: ' . $e->getMessage());
        }
    }
    
private function findApk($path)
{
    $files = scandir($path);
    foreach ($files as $file) {
        if (str_ends_with($file, '.apk')) {
            return $file;
        }
    }
    return null;
}


}
