<?php
namespace App\Http\Controllers;

use App\Models\Curso;
use Illuminate\Http\Request;

class CursoController extends Controller
{
    // Mostrar todos los cursos
    public function index()
    {
        $cursos = Curso::all();
        return view('cursos.index', compact('cursos'));
    }

    // Mostrar un curso específico
    public function show($id)
    {
        $curso = Curso::find($id);

        if (!$curso) {
            return redirect()->route('cursos.index')->with('error', 'Curso no encontrado');
        }

        return view('cursos.show', compact('curso'));
    }

    // Crear un nuevo curso
    public function create()
    {
        return view('cursos.create');
    }

    public function store(Request $request)
    {
        // Validaciones
        $validated = $request->validate([
            'nombre' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|unique:cursos,nombre|max:15',  // Validación única para el nombre
            'abreviacion' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|max:15',
            'nivel' => 'nullable|in:primaria,secundaria,universitaria', // Predeterminado primaria
        ], [
            'nombre.required' => 'El nombre del curso es obligatorio.',
            'nombre.unique' => 'El nombre del curso ya está registrado.',
            'nombre.regex' => 'El nombre solo puede contener letras, números, espacios y puntos.',
            'abreviacion.required' => 'La abreviatura del curso es obligatoria.',
            'abreviacion.regex' => 'La abreviatura solo puede contener letras, números, espacios y puntos.',
            'nivel.in' => 'El nivel debe ser uno de los siguientes: primaria, secundaria, universitaria.',
        ]);
    
        // Si no se especifica nivel, se establecerá como 'primaria' por defecto
        $validated['nivel'] = $validated['nivel'] ?? 'primaria';
        
        // Crear el curso
        Curso::create($validated);
    
        return redirect()->route('cursos.index')->with('success', 'Curso creado exitosamente');
    }
    
    // Mostrar el formulario de edición de un curso
    public function edit($id)
    {
        $curso = Curso::find($id);

        if (!$curso) {
            return redirect()->route('cursos.index')->with('error', 'Curso no encontrado');
        }

        return view('cursos.edit', compact('curso'));
    }

    // Actualizar un curso existente
    public function update(Request $request, $id)
    {
        $curso = Curso::find($id);
    
        if (!$curso) {
            return redirect()->route('cursos.index')->with('error', 'Curso no encontrado');
        }
    
        // Validaciones con la regla de unicidad para el nombre del curso, excluyendo el curso actual
        $validated = $request->validate([
            'nombre' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|max:15|unique:cursos,nombre,' . $id,
            'abreviacion' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|max:15',
            'nivel' => 'nullable|in:primaria,secundaria,universitaria',
        ], [
            'nombre.required' => 'El nombre del curso es obligatorio.',
            'nombre.regex' => 'El nombre solo puede contener letras, números, espacios y puntos.',
            'nombre.unique' => 'Ya existe un curso con ese nombre.',
            'abreviacion.required' => 'La abreviatura del curso es obligatoria.',
            'abreviacion.regex' => 'La abreviatura solo puede contener letras, números, espacios y puntos.',
            'nivel.in' => 'El nivel debe ser uno de los siguientes: primaria, secundaria, universitaria.',
        ]);
    
        // Si no se especifica nivel, se establecerá como 'primaria' por defecto
        $validated['nivel'] = $validated['nivel'] ?? 'primaria';
    
        // Actualizar el curso
        $curso->update($validated);
    
        return redirect()->route('cursos.index')->with('success', 'Curso actualizado exitosamente');
    }
    
    // Cambiar el estado del curso
    public function destroy($id)
    {
        $curso = Curso::find($id);

        if (!$curso) {
            return redirect()->route('cursos.index')->with('error', 'Curso no encontrado');
        }

        // Cambiar el estado (de 1 a 0 o viceversa)
        $curso->estado = $curso->estado == 1 ? 0 : 1;
        $curso->save();

        return redirect()->route('cursos.index')->with('success', 'Estado del curso cambiado exitosamente');
    }
}
