<?php

namespace App\Http\Controllers;

use App\Models\Curso;
use App\Models\Estudiantes;
use App\Models\User;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Http;
use Kreait\Firebase\JWT\CustomTokenGenerator;
use Kreait\Firebase\JWT\Action\CreateCustomToken;
class EstudianteController extends Controller
{
    function _construct(){
        

        $this->middleware('permission:ver-estudiante|crear-estudiante|editar-estudiante|borrar-estudiante',['only'=>['index']]);
        $this->middleware('permission:crear-estudiante',['only'=>['create','store']]);
        $this->middleware('permission:editar-estudiante',['only'=>['edit','update']]);
        $this->middleware('permission:borrar-estudiante',['only'=>['destroy']]);
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        $estudiantes = Estudiantes::simplePaginate(5);
        return view('estudiantes.index',compact('estudiantes'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $year = date('Y');
        $minYear = $year - 14;
        $maxYear = $year - 8;
    
        // Obtener todos los cursos disponibles
        $cursos = Curso::all();
    
        return view('estudiantes.crear', [
            'minYear' => $minYear,
            'maxYear' => $maxYear,
            'cursos' => $cursos // Pasar los cursos a la vista
        ]);
    }
    
    
    public function buscar(Request $request)
    {
        $query = $request->input('query');

        // Realiza la búsqueda en la base de datos
        $estudiantes = Estudiantes::where('CI', 'LIKE', '%' . $query . '%')->paginate(10);

        return view('estudiantes.index', compact('estudiantes'));
    }
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $messages = [
            'nombre.required' => 'El campo nombre es obligatorio.',
            'nombre.min' => 'El campo nombre debe tener al menos 3 caracteres.',
            'apellidos.required' => 'El campo apellidos es obligatorio.',
            'apellidos.min' => 'El campo apellidos debe tener al menos 5 caracteres.',
            'direccion.required' => 'El campo dirección es obligatorio.',
            'direccion.min' => 'El campo dirección debe tener al menos 10 caracteres.',
            'direccion.regex' => 'El campo dirección solo puede contener letras, números y los caracteres # y @.',
            'CI.required' => 'El campo CI es obligatorio.',
            'CI.numeric' => 'El campo CI debe ser numérico.',
            'CI.min' => 'El campo CI debe tener al menos 6 dígitos.',
            'CI.unique' => 'El CI ingresado ya está registrado.',
            'fecha.required' => 'El campo fecha es obligatorio.',
            'fecha.date' => 'El campo fecha debe ser una fecha válida.',
            'tel.required' => 'El campo teléfono es obligatorio.',
            'tel.numeric' => 'El campo teléfono debe ser numérico.',
            'tel.digits' => 'El campo teléfono debe tener 8 dígitos.',
            'correo.required' => 'El campo correo es obligatorio.',
            'correo.min' => 'El campo correo debe contener un mínimo de 11 caracteres.',
            'correo.regex' => 'El campo correo solo puede contener letras, números, @ y puntos.',
            'correo.email' => 'El campo correo debe ser una dirección de correo válida.',
            'correo.unique' => 'El correo electrónico ingresado ya está registrado.',
            'curso_id.required' => 'El campo curso es obligatorio.',
            'fecha.before_or_equal' => 'La fecha de nacimiento debe ser al menos 8 años anterior a la fecha actual.',
        ];
    
        $request->validate([
            'nombre' => 'required|string|min:3|max:30',
            'apellidos' => 'required|string|min:5|max:30',
            'condicion' => 'nullable|string|min:7|max:50',
            'CI' => 'required|numeric|min:6|unique:estudiantes,CI',
            'fecha' => 'required|date|before_or_equal:' . \Carbon\Carbon::now()->subYears(8)->format('Y-m-d'),
            'tel' => 'required|numeric|digits:8',
            'direccion' => 'required|string|min:10|regex:/^[a-zA-Z0-9\s#]+$/',
            'correo' => 'required|string|min:11|regex:/^[a-zA-Z0-9\-_.]+@[a-zA-Z0-9-]+\.[a-zA-Z]{2,}$/|email|unique:estudiantes,correo',
            'curso_id' => 'required|exists:cursos,id',  // Validar que el curso exista
        ], $messages);
    
        // Establecer 'condicion' a 'Ninguna' si está vacío
        $validatedData = $request->all();
        if (empty($validatedData['condicion'])) {
            $validatedData['condicion'] = 'Ninguna';
        }
    
        // Asignar estado 1 por defecto
        $validatedData['estado'] = 1;
    
        // Crear el estudiante
        Estudiantes::create($validatedData);
    
        $notificacion = 'El estudiante se agregó correctamente.';
        return redirect()->route('estudiantes.index')->with(compact('notificacion'));
    }
    
    

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Estudiantes $estudiante)
    {
        // Obtener todos los cursos disponibles
        $cursos = Curso::all();
    
        return view('estudiantes.editar', compact('estudiante', 'cursos'));
    }
    
    public function search(Request $request)
    {
        $searchQuery = $request->input('query');
        
        $estudiantes = Estudiantes::where('CI', 'LIKE', "%{$searchQuery}%")->paginate(5);
        
        return view('estudiantes.index', compact('estudiantes'));
    }
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Estudiantes $estudiante)
    {
        $messages = [
            'required' => 'El campo :attribute es obligatorio.',
            'numeric' => 'El campo :attribute debe ser un número.',
            'date' => 'El campo :attribute debe ser una fecha válida.',
            'digits' => 'El campo :attribute debe tener :digits dígitos.',
            'regex' => 'El campo :attribute no cumple con el formato válido.',
            'email' => 'El campo :attribute debe ser una dirección de correo válida.',
            'unique' => 'El :attribute ingresado ya está registrado.',
            'fecha.before_or_equal' => 'La fecha de nacimiento debe ser al menos 8 años anterior a la fecha actual.',
        ];
    
        // Validación de los datos
        $request->validate([
            'nombre' => 'required|string|min:3|max:30',
            'apellidos' => 'required|string|min:5|max:30',
            'condicion' => 'nullable|string|min:7|max:50',
            'CI' => 'required|numeric|min:6|unique:estudiantes,CI,' . $estudiante->id,
            'fecha' => 'required|date|before_or_equal:' . \Carbon\Carbon::now()->subYears(8)->format('Y-m-d'),
            'tel' => 'required|numeric|digits:8',
            'direccion' => 'required|string|min:10|regex:/^[a-zA-Z0-9\s#]+$/',
            'correo' => 'required|string|min:11|email|regex:/^[a-zA-Z0-9\-_.]+@[a-zA-Z0-9-]+\.[a-zA-Z]{2,}$/|unique:estudiantes,correo,' . $estudiante->id,
            'curso_id' => 'required|exists:cursos,id', // Asegurarse de que el curso existe
        ], $messages);
    
        // Establecer 'condicion' a 'Ninguna' si está vacío
        $validatedData = $request->all();
        if (empty($validatedData['condicion'])) {
            $validatedData['condicion'] = 'Ninguna';
        }
    
        // Actualizar el estudiante
        $estudiante->update($validatedData);
    
        $notificacion = 'El estudiante se actualizó correctamente.';
        return redirect()->route('estudiantes.index')->with(compact('notificacion'));
    }
    

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Estudiantes $estudiante)
    {
        $deleteName = $estudiante->nombre;
        
        // Cambiar el estado del estudiante (de 1 a 0, o de 0 a 1)
        $estudiante->estado = $estudiante->estado == 1 ? 0 : 1;
        $estudiante->save();
        
        // Notificación
        $notificacion = 'El estudiante ' . $deleteName . ' se ' . ($estudiante->estado == 1 ? 'activó' : 'desactivó') . ' correctamente.';
        return redirect()->route('estudiantes.index')->with(compact('notificacion'));
    }
    
       
    public function generateUserForm($estudianteId)
    {
        $estudiante = Estudiantes::find($estudianteId);
        
        // Verificar si  existe
        if (!$estudiante) {
            abort(404, 'estudiante no encontrado');
        }
        
        return view('estudiantes.generate_user', compact('estudiante'));
    }

    protected $firestoreUrl;
protected $projectId;
protected $privateKey;
protected $clientEmail;

public function __construct()
{
    // Datos de configuración para Firebase
    $this->projectId = 'integrador-9f039';
    $this->firestoreUrl = 'https://firestore.googleapis.com/v1/projects/' . $this->projectId . '/databases/(default)/documents';
    $this->privateKey = "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQC2/JFdt39wDoZd\np2+K1grHTDn73il0LGzuiJ387/t/eGNKe2EMUKcFo9UKzg7c0DFn+ljjSIzMMbId\n3CuHdtO/PWJFyEQ2DmAdAs7JF9gsaWF7a+PuH+AwqFOwC+y63oEqVCD+ZVILSwvY\nerZynnf0+VOP/EyUW7lS+B3tJ/9yRgiQ5tVRYr6MgEJaBhr3c0f7LsNkJFC50x9G\nyyG5hSbsPf6TaJXTsM8EkNbPkLz2fITUykSbDzVjNfh0avhZohcNiThEGO31n9Hs\ngm33C4cExC/ke5l8OmK+2Hy2Qy2MgXE48/yUdf+QVsD96XWv9VPOfUClm0ei0oWA\nUwY4UY7TAgMBAAECggEAFVTsPVie6plG4LQcNLsvFny0ZcBIZNqo+EXE5S+0ye9m\n48/uSFk+n1GyDEhpQySiHdT5d8WFZGiGsyRvjqMznB+zzR7DlOmoGrmxEZBOCx9d\n1d2F/MSKhiGZX/SD7GTz+dNE5WM6ZUaiYuIrUsFWiqBmNWrcIo9u01VJZ9Q520BL\n89qhB+0HBNyzcy/q8bbQqMT/NPq+X+eCg7GA//qTLf3cYXtI6M8irlhFnWqmbKkn\n5S3SFJ7dfVgTmN30aZT2kqkp+lFsXXbYlW3yIrDjQaOuRavad9oeegvDjwLiJCP1\nEylWi3AQJPZFj+83m8eThfmwrmC/34AuYilcU+DoQQKBgQD1PXRNwwFSqwgzi641\nZUYAegZ8GzBasEZjn9I1vGMKb6v5x+fN4/d3wn3zV3u8kY8X02g+H1lpgX0gLxmg\nqj6pygs4qcGC83Y6gSJhyvoT3w4llX3NG/o+5orsGb/+caQJ6kJEAxa7uf1IUVkZ\nouD4mM7g4EFLo+q0V+b701SV5QKBgQC/A+FYBgBrA2NoEx9l6x1iUyEKovTrwx6/\nB5u/LDFI907sbFoZjvOpXqhVvcT9d/lWAf7d7IhsT3D48VyxO86lkcZ88IsywKXF\nHWsNfbncrewxIeK4Jai9ksy0+ZUbCG6nIsj8OkxhbISLf6MjqFCxm6NttThEXUMH\nfwPQld5GVwKBgF2j2YTXsOmf5MHJ0/sPcGs+LLu2pf/6zuW7YMij5i1V9pSpgBld\nsmBII0+HV/fonHq8flE6JyniB6UCVwbB+qtaOm6oYvtdewdbx4UPu+1afkxbiG2J\nAUSDQwhjzKubOMeQaz0M8OE9Yzj51suzJbPGpTe4aTY4oKW76ra9Os8BAoGAM3pa\nOpyzdCOfoKa2qYLpklwQ5YrxMrCNErJ8ONMyHRX5QSIaThUGebi26RSRiDXTDmjP\nRQO/6GCoc46oimMx19EnivhQq0bb5fl9pk8hzeVZaAxXy4TqqtxrKsKwem6VU5m+\nTKtxdjgGqImnb66mwJbPPnu+RD5WGCkU4BvjoVUCgYEAyaZZvfYdrY7RVwVBGfwf\nHQ/4qyEHslFcY2LbHMEwYT4cDEPBaBGxDbAAQ5udPBP+nvKHouVUwKGrul/OKd1c\nlU0cpZur8qdOU1rnqkPnO9H2wnsLaTdEXZt0k+/sK60Hg6o5ElXuubGP44EfdxQF\nD57oN0dNNfNcVZ0aAxbhOK8=\n-----END PRIVATE KEY-----\n";
    $this->clientEmail = 'firebase-adminsdk-oim2s@integrador-9f039.iam.gserviceaccount.com';
}


public function generateUser(Request $request, $estudianteId)
{
    $estudiante = Estudiantes::findOrFail($estudianteId);

    // Validar la solicitud
    $request->validate([
        'password' => 'required|string|min:8|confirmed',
    ]);

    try {
        // Verificar si el correo electrónico ya está registrado en Firestore
        $existingEmailQuery = Http::post($this->firestoreUrl . ':runQuery', [
            'structuredQuery' => [
                'from' => [['collectionId' => 'Users']],
                'where' => [
                    'fieldFilter' => [
                        'field' => ['fieldPath' => 'correo'],
                        'op' => 'EQUAL',
                        'value' => ['stringValue' => $estudiante->correo],
                    ],
                ],
                'limit' => 1
            ]
        ]);

        $existingEmail = $existingEmailQuery->json();

        if (isset($existingEmail[0]['document'])) {
            return redirect()->back()->with('error', 'El correo electrónico ya está registrado en Firestore.');
        }

        // Crear un nuevo usuario para el estudiante en la base de datos local (con el rol Estudiante)
        $user = User::create([
            'name' => $estudiante->nombre,
            'email' => $estudiante->correo,
            'password' => bcrypt($request->password),  // Cifrado localmente
            'estado' => $estudiante->estado
        ]);

        // Asignar el rol "Estudiante" al usuario
        $user->assignRole('Estudiante');

        // Guardar el ID del usuario en el estudiante
        $estudiante->user_id = $user->id;
        $estudiante->save();

        // Cifrar la contraseña para Firestore (Opcional: puedes enviarla como texto plano si realmente es necesario)
        $firestorePassword = base64_encode($request->password);

        // Crear el estudiante en Firestore
        $response = Http::post($this->firestoreUrl . '/Users', [
            'fields' => [
                'nombre' => ['stringValue' => $estudiante->nombre],
                'correo' => ['stringValue' => $estudiante->correo],
                'curso' => ['stringValue' => $estudiante->curso->nombre], // Guardar curso del estudiante
                'estado' => ['integerValue' => (int) $estudiante->estado],  // Convertir estado a entero
                'id_local' => ['stringValue' => (string) $estudiante->id], // Guardar el ID del estudiante local
                'password' => ['stringValue' => $firestorePassword], // Contraseña cifrada como base64
            ]
        ]);

        if ($response->successful()) {
            $notificacion = 'Usuario generado correctamente para ' . $estudiante->nombre . ' y guardado en Firestore';
            return redirect()->route('estudiantes.index')->with(compact('notificacion'));
        } else {
            Log::error('Error al enviar datos a Firestore', ['response' => $response->json()]);
            return redirect()->back()->with('error', 'Error al crear el usuario en Firestore.');
        }
    } catch (\Exception $e) {
        Log::error('Error al crear el usuario.', ['exception' => $e->getMessage()]);
        return redirect()->back()->with('error', 'Error al crear el usuario. Detalles: ' . $e->getMessage());
    }
}

}