<?php

namespace App\Http\Controllers;

use App\Models\Maestro;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

class MaestroController extends Controller
{
   
    public function index()
    {
        $maestros = Maestro::paginate(5);
        return view('maestros.index', compact('maestros'));
    }

    public function create()
    {
        $year = date('Y');
        $minYear = $year - 50;
        $maxYear = $year - 25;

        return view('maestros.crear', compact('year', 'minYear', 'maxYear'));
    }

    public function store(Request $request)
    {
        $messages = [
            'required' => 'El campo :attribute es obligatorio.',
            'numeric' => 'El campo :attribute debe ser un número.',
            'date' => 'El campo :attribute debe ser una fecha válida.',
            'digits' => 'El campo :attribute debe tener :digits dígitos.',
            'regex' => 'El campo :attribute no cumple con el formato válido.',
            'email' => 'El campo :attribute debe ser una dirección de correo válida.',
            'unique' => 'El carnet ya está registrado.',
        ];

        $year = date('Y');
        $minYear = $year - 50;
        $maxYear = $year - 25;

        $request->validate([
            'nombre' => 'required',
            'apellidos' => 'required',
            'CI' => 'required|numeric|unique:maestros,CI',
            'fecha' => ['required', 'date', 'before:' . $maxYear . '-12-31', 'after:' . $minYear . '-01-01'],
            'tel' => 'required|numeric|digits:8',
            'direccion' => ['required', 'regex:/^[a-zA-Z0-9\s#]+$/'],
            'correo' => 'required|email',
            'especialidad' => 'required|regex:/^[a-zA-Z]+$/',
        ], $messages);

        // Crear maestro con estado por defecto 1 (activo)
        Maestro::create(array_merge($request->all(), ['estado' => 1]));

        $notificacion = 'El maestro se agregó correctamente.';
        return redirect()->route('maestros.index')->with(compact('notificacion'));
    }

    public function destroy(Maestro $maestro)
    {
        // Alternar el estado entre activo (1) e inactivo (0)
        $maestro->estado = $maestro->estado ? 0 : 1;
        $maestro->save();

        $estado = $maestro->estado ? 'activado' : 'desactivado';
        $notificacion = "El estado del maestro {$maestro->nombre} se ha {$estado} correctamente.";
        return redirect()->route('maestros.index')->with(compact('notificacion'));
    }

    public function edit(Maestro $maestro)
    {
        $year = date('Y');
        $minYear = $year - 50;
        $maxYear = $year - 25;

        return view('maestros.editar', compact('year', 'minYear', 'maxYear', 'maestro'));
    }

    public function update(Request $request, Maestro $maestro)
    {
        $messages = [
            'required' => 'El campo :attribute es obligatorio.',
            'numeric' => 'El campo :attribute debe ser un número.',
            'date' => 'El campo :attribute debe ser una fecha válida.',
            'digits' => 'El campo :attribute debe tener :digits dígitos.',
            'regex' => 'El campo :attribute no cumple con el formato válido.',
            'email' => 'El campo :attribute debe ser una dirección de correo válida.',
        ];

        $year = date('Y');
        $minYear = $year - 50;
        $maxYear = $year - 25;

        $request->validate([
            'nombre' => 'required',
            'apellidos' => 'required',
            'CI' => 'required|numeric',
            'fecha' => ['required', 'date', 'before:' . $maxYear . '-12-31', 'after:' . $minYear . '-01-01'],
            'tel' => 'required|numeric|digits:8',
            'direccion' => ['required', 'regex:/^[a-zA-Z0-9\s#]+$/'],
            'correo' => 'required|email',
            'especialidad' => 'required|regex:/^[a-zA-Z]+$/',
        ], $messages);

        $maestro->update($request->all());
        $notificacion = 'El maestro se actualizó correctamente.';
        return redirect()->route('maestros.index')->with(compact('notificacion'));
    }

    public function generateUserForm(Maestro $maestro)
    {
        // Retornar la vista para generar usuario con el maestro seleccionado
        return view('maestros.generate_user', compact('maestro'));
    }
    
    public function generateUser(Maestro $maestro, Request $request)
    {
        $request->validate([
            'password' => 'required|string|min:8|confirmed',
        ]);
    
        try {
            // Verificar si el usuario ya existe
            if (User::where('email', $maestro->correo)->exists()) {
                return redirect()->back()->with('error', 'El usuario ya existe.');
            }
    
            // Crear un nuevo usuario con rol Maestro
            $user = User::create([
                'name' => $maestro->nombre . ' ' . $maestro->apellidos,
                'email' => $maestro->correo,
                'password' => Hash::make($request->password),
                'estado' => $maestro->estado,
            ]);
    
            // Asignar el rol "Maestro"
            $user->assignRole('Maestro');
    
            // Actualizar el campo user_id en la tabla maestros
            $maestro->update([
                'user_id' => $user->id, // Vincular el user_id recién creado
            ]);
    
            return redirect()->route('maestros.index')->with('success', 'Usuario generado correctamente con el rol de Maestro.');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Error al crear el usuario: ' . $e->getMessage());
        }
    }
    
}
