<?php

namespace App\Http\Controllers;

use App\Models\Curso;
use App\Models\Materia;
use Illuminate\Http\Request;

class MateriaController extends Controller
{
    public function index()
    {
        $materias = Materia::all(); // Aquí puedes filtrar las materias activas si es necesario
        return view('materias.index', compact('materias'));
    }
    
    // Mostrar una materia específica
    public function show($id)
    {
        $materia = Materia::find($id);

        if (!$materia) {
            return redirect()->route('materias.index')->with('error', 'Materia no encontrada');
        }

        return view('materias.show', compact('materia'));
    }

    // Crear una nueva materia
    public function create()
    {
        $cursos = Curso::where('estado', 1)->get(); // Obtener solo los cursos activos
        return view('materias.create', compact('cursos'));
    }
    

    public function store(Request $request)
    {
        // Validaciones
        $validated = $request->validate([
            'nombre' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|max:50|unique:materias,nombre,NULL,id,curso_id,' . $request->curso_id,
            'abreviacion' => 'nullable|string|max:10',
            'curso_id' => 'required|exists:cursos,id',
        ], [
            'nombre.required' => 'El nombre de la materia es obligatorio.',
            'nombre.unique' => 'El nombre de la materia ya está registrado para este curso.',
            'nombre.regex' => 'El nombre solo puede contener letras, números, espacios y puntos.',
            'abreviacion.max' => 'La abreviatura no puede exceder los 10 caracteres.',
            'curso_id.required' => 'El curso es obligatorio.',
            'curso_id.exists' => 'El curso seleccionado no existe.',
        ]);
    
        // Si no se proporciona abreviatura, generarla automáticamente
        if (!$request->abreviacion) {
            $curso = Curso::find($request->curso_id);
            $abreviacion = strtoupper(substr($request->nombre, 0, 3)) . '-' . strtoupper(substr($curso->nombre, 0, 5)). '-' . strtoupper(substr($curso->abreviacion, 0, 6)); // Ejemplo: HIS-PRI
    
            // Comprobar si la abreviatura ya existe
            $counter = 1;
            $originalAbreviacion = $abreviacion;
            while (Materia::where('abreviacion', $abreviacion)->exists()) {
                $abreviacion = $originalAbreviacion . '-' . $counter++;
            }
    
            $validated['abreviacion'] = $abreviacion;
        }
    
        // Asignar el estado por defecto como 1
        $validated['estado'] = $validated['estado'] ?? 1;
    
        // Crear la materia
        Materia::create($validated);
    
        return redirect()->route('materias.index')->with('success', 'Materia creada exitosamente');
    }
    
    
    
    

    // Mostrar el formulario de edición
    public function edit($id)
    {
        $materia = Materia::find($id);
    
        if (!$materia) {
            return redirect()->route('materias.index')->with('error', 'Materia no encontrada');
        }
    
        // Obtener solo los cursos activos
        $cursos = Curso::where('estado', 1)->get();
    
        return view('materias.edit', compact('materia', 'cursos'));
    }
    
    public function update(Request $request, $id)
    {
        $materia = Materia::find($id);
    
        if (!$materia) {
            return redirect()->route('materias.index')->with('error', 'Materia no encontrada');
        }
    
        // Validaciones
        $validated = $request->validate([
            'nombre' => 'required|string|regex:/^[A-Za-z0-9 .]+$/|max:50|unique:materias,nombre,' . $materia->id . ',id,curso_id,' . $request->curso_id,
            'abreviacion' => 'nullable|string|max:10',
            'curso_id' => 'required|exists:cursos,id',
        ], [
            'nombre.required' => 'El nombre de la materia es obligatorio.',
            'nombre.unique' => 'El nombre de la materia ya está registrado para este curso.',
            'nombre.regex' => 'El nombre solo puede contener letras, números, espacios y puntos.',
            'abreviacion.max' => 'La abreviatura no puede exceder los 10 caracteres.',
            'curso_id.required' => 'El curso es obligatorio.',
            'curso_id.exists' => 'El curso seleccionado no existe.',
        ]);
    
        // Si no se proporciona abreviatura, generarla automáticamente
        if (!$request->abreviacion) {
            $curso = Curso::find($request->curso_id);
            $abreviacion = strtoupper(substr($request->nombre, 0, 3)) . '-' . strtoupper(substr($curso->nombre, 0, 3)); // Ejemplo: HIS-PRI
    
            // Comprobar si la abreviatura ya existe
            $counter = 1;
            $originalAbreviacion = $abreviacion;
            while (Materia::where('abreviacion', $abreviacion)->exists()) {
                $abreviacion = $originalAbreviacion . '-' . $counter++;
            }
    
            $validated['abreviacion'] = $abreviacion;
        }
    
        // Asignar el estado por defecto como 1 (si no se está pasando en la actualización)
        $validated['estado'] = $validated['estado'] ?? 1;
    
        // Actualizar la materia
        $materia->update($validated);
    
        return redirect()->route('materias.index')->with('success', 'Materia actualizada exitosamente');
    }
    
    // Cambiar el estado de la materia
    public function destroy($id)
    {
        $materia = Materia::find($id);

        if (!$materia) {
            return redirect()->route('materias.index')->with('error', 'Materia no encontrada');
        }

        // Cambiar el estado (de 1 a 0 o viceversa)
        $materia->estado = $materia->estado == 1 ? 0 : 1;
        $materia->save();

        return redirect()->route('materias.index')->with('success', 'Estado de la materia cambiado exitosamente');
    }
}
