<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Modelo;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Http;
class ModeloController extends Controller
{
    public function index()
    {
        $modelos = Modelo::paginate(10);
        return view('modelo.index', compact('modelos'));
    }

    public function create()
    {
        return view('modelo.create');
    }

    public function store(Request $request)
    {
        // Validar los datos ingresados
        $request->validate([
            'nombre' => 'required|string|max:255',
            'descripcion' => 'nullable|string',
            'archivo_zip' => 'required|file|mimes:zip|max:40048', // Validar archivo ZIP
        ]);
    
        try {
            \Log::info('Iniciando el proceso de creación del modelo.');
    
            // Subir el archivo ZIP
            if ($request->hasFile('archivo_zip')) {
                $path = $request->file('archivo_zip')->store('modelos', 'public');
                \Log::info('Archivo ZIP subido correctamente.', ['path' => $path]);
    
                // Ruta del directorio donde se descomprimirá
                $extractPath = 'modelos_descomprimidos/' . uniqid();
    
                // Descomprimir el archivo ZIP
                $zip = new \ZipArchive();
                if ($zip->open(storage_path('app/public/' . $path)) === true) {
                    $zip->extractTo(public_path($extractPath));
                    $zip->close();
                    \Log::info('Archivo ZIP descomprimido correctamente.', ['extractPath' => $extractPath]);
                } else {
                    return redirect()->back()->with('error', 'No se pudo descomprimir el archivo.');
                }
            } else {
                return redirect()->back()->with('error', 'Error al subir el archivo.');
            }
    
            // Crear un nuevo modelo con estado por defecto 1
            $modelo = Modelo::create([
                'nombre' => $request->nombre,
                'descripcion' => $request->descripcion,
                'archivo_zip' => $path, // Guardar la ruta del archivo ZIP
                'carpeta_descomprimida' => $extractPath,
                'url' => asset($extractPath . '/index.html'), // Generar la URL al index.html
                'estado' => 1, // Activo por defecto
            ]);
    
            \Log::info('Modelo creado exitosamente en la base de datos.');
            return redirect()->route('modelo.index')->with('success', 'Modelo creado correctamente.');
        } catch (\Exception $e) {
            \Log::error('Error al crear el modelo.', ['exception' => $e->getMessage()]);
            return redirect()->back()->with('error', 'Ocurrió un error al crear el modelo. Detalles: ' . $e->getMessage());
        }
    }
    
    public function edit(Modelo $modelo)
    {
        return view('modelo.edit', compact('modelo'));
    }

    public function update(Request $request, Modelo $modelo)
    {
        // Validar los datos ingresados
        $request->validate([
            'nombre' => 'required|string|max:255',
            'descripcion' => 'nullable|string',
            'archivo_zip' => 'nullable|file|mimes:zip|max:40048', // Archivo opcional
        ]);

        try {
            Log::info('Iniciando el proceso de actualización del modelo.', ['modelo_id' => $modelo->id]);

            // Subir el nuevo archivo si existe
            if ($request->hasFile('archivo_zip')) {
                // Eliminar el archivo anterior
                if ($modelo->archivo_zip) {
                    Storage::disk('public')->delete($modelo->archivo_zip);
                    Log::info('Archivo ZIP anterior eliminado.', ['path' => $modelo->archivo_zip]);
                }

                $path = $request->file('archivo_zip')->store('modelos', 'public');
                $modelo->archivo_zip = $path;
                Log::info('Archivo ZIP actualizado.', ['path' => $path]);
            }

            // Actualizar los datos del modelo
            $modelo->nombre = $request->nombre;
            $modelo->descripcion = $request->descripcion;
            $modelo->save();

            Log::info('Modelo actualizado exitosamente.');
            return redirect()->route('modelo.index')->with('success', 'Modelo actualizado correctamente.');
        } catch (\Exception $e) {
            Log::error('Error al actualizar el modelo.', ['exception' => $e->getMessage()]);
            return redirect()->back()->with('error', 'Ocurrió un error al actualizar el modelo. Detalles: ' . $e->getMessage());
        }
    }

    public function destroy(Modelo $modelo)
    {
        try {
            Log::info('Iniciando el proceso de cambio de estado del modelo.', ['modelo_id' => $modelo->id]);

            // Cambiar el estado del modelo (de 1 a 0 y viceversa)
            $modelo->estado = !$modelo->estado;
            $modelo->save();

            Log::info('Estado del modelo actualizado.', ['nuevo_estado' => $modelo->estado]);
            return redirect()->route('modelo.index')->with('success', 'Estado del modelo actualizado correctamente.');
        } catch (\Exception $e) {
            Log::error('Error al cambiar el estado del modelo.', ['exception' => $e->getMessage()]);
            return redirect()->back()->with('error', 'Ocurrió un error al actualizar el estado. Detalles: ' . $e->getMessage());
        }
    }


    protected $firestoreUrl;
    protected $projectId;
    protected $privateKey;
    protected $clientEmail;

    public function __construct()
    {
        // Datos de configuración para Firebase
        $this->projectId = 'integrador-9f039';
        $this->firestoreUrl = 'https://firestore.googleapis.com/v1/projects/' . $this->projectId . '/databases/(default)/documents';
        $this->privateKey = "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQC2/JFdt39wDoZd\np2+K1grHTDn73il0LGzuiJ387/t/eGNKe2EMUKcFo9UKzg7c0DFn+ljjSIzMMbId\n3CuHdtO/PWJFyEQ2DmAdAs7JF9gsaWF7a+PuH+AwqFOwC+y63oEqVCD+ZVILSwvY\nerZynnf0+VOP/EyUW7lS+B3tJ/9yRgiQ5tVRYr6MgEJaBhr3c0f7LsNkJFC50x9G\nyyG5hSbsPf6TaJXTsM8EkNbPkLz2fITUykSbDzVjNfh0avhZohcNiThEGO31n9Hs\ngm33C4cExC/ke5l8OmK+2Hy2Qy2MgXE48/yUdf+QVsD96XWv9VPOfUClm0ei0oWA\nUwY4UY7TAgMBAAECggEAFVTsPVie6plG4LQcNLsvFny0ZcBIZNqo+EXE5S+0ye9m\n48/uSFk+n1GyDEhpQySiHdT5d8WFZGiGsyRvjqMznB+zzR7DlOmoGrmxEZBOCx9d\n1d2F/MSKhiGZX/SD7GTz+dNE5WM6ZUaiYuIrUsFWiqBmNWrcIo9u01VJZ9Q520BL\n89qhB+0HBNyzcy/q8bbQqMT/NPq+X+eCg7GA//qTLf3cYXtI6M8irlhFnWqmbKkn\n5S3SFJ7dfVgTmN30aZT2kqkp+lFsXXbYlW3yIrDjQaOuRavad9oeegvDjwLiJCP1\nEylWi3AQJPZFj+83m8eThfmwrmC/34AuYilcU+DoQQKBgQD1PXRNwwFSqwgzi641\nZUYAegZ8GzBasEZjn9I1vGMKb6v5x+fN4/d3wn3zV3u8kY8X02g+H1lpgX0gLxmg\nqj6pygs4qcGC83Y6gSJhyvoT3w4llX3NG/o+5orsGb/+caQJ6kJEAxa7uf1IUVkZ\nouD4mM7g4EFLo+q0V+b701SV5QKBgQC/A+FYBgBrA2NoEx9l6x1iUyEKovTrwx6/\nB5u/LDFI907sbFoZjvOpXqhVvcT9d/lWAf7d7IhsT3D48VyxO86lkcZ88IsywKXF\nHWsNfbncrewxIeK4Jai9ksy0+ZUbCG6nIsj8OkxhbISLf6MjqFCxm6NttThEXUMH\nfwPQld5GVwKBgF2j2YTXsOmf5MHJ0/sPcGs+LLu2pf/6zuW7YMij5i1V9pSpgBld\nsmBII0+HV/fonHq8flE6JyniB6UCVwbB+qtaOm6oYvtdewdbx4UPu+1afkxbiG2J\nAUSDQwhjzKubOMeQaz0M8OE9Yzj51suzJbPGpTe4aTY4oKW76ra9Os8BAoGAM3pa\nOpyzdCOfoKa2qYLpklwQ5YrxMrCNErJ8ONMyHRX5QSIaThUGebi26RSRiDXTDmjP\nRQO/6GCoc46oimMx19EnivhQq0bb5fl9pk8hzeVZaAxXy4TqqtxrKsKwem6VU5m+\nTKtxdjgGqImnb66mwJbPPnu+RD5WGCkU4BvjoVUCgYEAyaZZvfYdrY7RVwVBGfwf\nHQ/4qyEHslFcY2LbHMEwYT4cDEPBaBGxDbAAQ5udPBP+nvKHouVUwKGrul/OKd1c\nlU0cpZur8qdOU1rnqkPnO9H2wnsLaTdEXZt0k+/sK60Hg6o5ElXuubGP44EfdxQF\nD57oN0dNNfNcVZ0aAxbhOK8=\n-----END PRIVATE KEY-----\n";
        $this->clientEmail = 'firebase-adminsdk-oim2s@integrador-9f039.iam.gserviceaccount.com';
    }


    public function generateQr($id)
    {
        try {
            // Buscar el modelo
            $modelo = Modelo::findOrFail($id);
    
            // Generar la URL del modelo
            $modelo->url = 'http://192.168.1.200:8000/storage/' . $modelo->carpeta_descomprimida . '/index.html';
    
            // Generar el código QR (Base64)
            $qrCode = \SimpleSoftwareIO\QrCode\Facades\QrCode::format('svg') // Cambia a SVG
                ->size(300)
                ->generate($modelo->url);
            $qrBase64 = base64_encode($qrCode);
    
            // Verificar si el usuario tiene el rol de Administrador
            if (auth()->user()->hasRole('Administrador')) {
                // Datos para Firestore
                $data = [
                    'fields' => [
                        'nombre' => ['stringValue' => $modelo->nombre],
                        'descripcion' => ['stringValue' => $modelo->descripcion],
                        'url' => ['stringValue' => $modelo->url],
                        'codigo_qr' => ['stringValue' => $qrBase64], // Almacenar QR en formato base64
                        'estado' => ['integerValue' => $modelo->estado],
                        'id_local' => ['stringValue' => (string) $modelo->id],
                    ],
                ];
    
                // Enviar datos a Firestore
                $response = Http::post($this->firestoreUrl . '/modelos', [
                    'fields' => $data['fields']
                ]);
    
                // Verificar respuesta de Firestore
                if ($response->successful()) {
                    Log::info('Código QR almacenado en Firestore correctamente.', ['id' => $modelo->id]);
                    return view('modelo.qr', compact('modelo', 'qrCode'))
                        ->with('success', 'Código QR generado y almacenado en Firestore.');
                } else {
                    Log::error('Error al enviar datos a Firestore', ['response' => $response->json()]);
                    return redirect()->back()->with('error', 'Error al almacenar el código QR en Firestore.');
                }
            }
    
            // Para otros roles, solo mostrar el QR
            return view('modelo.qr', compact('modelo', 'qrCode'))
                ->with('success', 'Código QR generado correctamente.');
    
        } catch (\Exception $e) {
            Log::error('Error al generar QR: ' . $e->getMessage());
            return redirect()->back()->with('error', 'Error al generar QR: ' . $e->getMessage());
        }
    }
    

    public function descomprimir($id)
{
    try {
        $modelo = Modelo::findOrFail($id);

        if (!$modelo->archivo_zip) {
            return redirect()->back()->with('error', 'El modelo no tiene un archivo ZIP asignado.');
        }

        // Ruta del archivo ZIP
        $zipPath = storage_path('app/public/' . $modelo->archivo_zip);

        // Nueva ruta pública donde se descomprimirá
        $extractPath = public_path('modelos/' . $modelo->id);

        // Crear la carpeta si no existe
        if (!is_dir($extractPath)) {
            mkdir($extractPath, 0755, true);
        }

        // Descomprimir el archivo ZIP
        $zip = new \ZipArchive;
        if ($zip->open($zipPath) === TRUE) {
            $zip->extractTo(public_path('storage/modelos/' . $modelo->id));
            $zip->close();

            // Actualizar la URL en la base de datos
            $modelo->carpeta_descomprimida = 'modelos/' . $modelo->id;
            $modelo->url = asset('modelos/' . $modelo->id . '/index.html'); // Generar URL pública
            $modelo->save();

            \Log::info('Modelo descomprimido correctamente.', ['modelo_id' => $modelo->id]);
            return redirect()->back()->with('success', 'Modelo descomprimido y URL generada correctamente.');
        } else {
            throw new \Exception('No se pudo abrir el archivo ZIP.');
        }
    } catch (\Exception $e) {
        \Log::error('Error al descomprimir el modelo.', ['exception' => $e->getMessage()]);
        return redirect()->back()->with('error', 'Error al descomprimir el modelo. Detalles: ' . $e->getMessage());
    }
}



}
