<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\Maestro;

class ProgresoController extends Controller
{
    protected $firestoreUrl;

    public function __construct()
    {
        $this->firestoreUrl = 'https://firestore.googleapis.com/v1/projects/integrador-9f039/databases/(default)/documents/progreso';
    }

    public function index()
    {
        try {
            Log::info('Inicio de carga del progreso estudiantil.');
    
            // Obtener el usuario autenticado
            $user = Auth::user();
    
            // Obtener parámetros de búsqueda
            $search = request('search'); // Búsqueda general
            $startDate = request('start_date'); // Fecha de inicio
            $endDate = request('end_date'); // Fecha de fin
    
            // Verificar si es Administrador: mostrar todo
            if ($user->hasRole('Administrador')) {
                $progresos = $this->mostrarTodoProgreso($search, $startDate, $endDate);
                return view('progreso.index', ['progresos' => $progresos]);
            }
    
            // Verificar si es Maestro
            if ($user->hasRole('Maestro')) {
                $maestro = Maestro::where('user_id', $user->id)->first();
    
                if (!$maestro) {
                    throw new \Exception("No se encontró el maestro asociado al usuario.");
                }
    
                // Obtener cursos desde sus materias
                $cursosAsignados = $maestro->materias()
                    ->with('curso')
                    ->get()
                    ->pluck('curso.nombre')
                    ->unique();
    
                $progresos = $this->filtrarProgresoPorCurso($cursosAsignados, $search, $startDate, $endDate);
                return view('progreso.index', ['progresos' => $progresos]);
            }
    
            return redirect()->back()->with('error', 'No tiene permisos para acceder.');
        } catch (\Exception $e) {
            Log::error('Error al cargar progreso estudiantil.', ['error' => $e->getMessage()]);
            return redirect()->back()->with('error', 'Error al cargar el progreso estudiantil: ' . $e->getMessage());
        }
    }
    
    /**
     * Mostrar progreso filtrado para Administrador o Maestro.
     */
    private function mostrarTodoProgreso($search, $startDate, $endDate)
    {
        $response = Http::get($this->firestoreUrl);
    
        if (!$response->successful()) {
            throw new \Exception("Error al obtener datos de Firestore.");
        }
    
        return $this->aplicarFiltros($response->json()['documents'] ?? [], $search, $startDate, $endDate);
    }
    
    /**
     * Filtrar progreso por curso asignado y parámetros adicionales.
     */
    private function filtrarProgresoPorCurso($cursosAsignados, $search, $startDate, $endDate)
    {
        $response = Http::get($this->firestoreUrl);
    
        if (!$response->successful()) {
            throw new \Exception("Error al obtener datos de Firestore.");
        }
    
        $progresos = $this->aplicarFiltros($response->json()['documents'] ?? [], $search, $startDate, $endDate);
    
        // Filtrar por cursos asignados
        return $progresos->filter(function ($progreso) use ($cursosAsignados) {
            return $cursosAsignados->contains($progreso['usuario_curso']);
        });
    }
    
    /**
     * Aplicar filtros de búsqueda y fecha.
     */
    private function aplicarFiltros($documents, $search, $startDate, $endDate)
    {
        $progresos = collect($documents)->map(function ($doc) {
            $fields = $doc['fields'] ?? [];
            return [
                'modelo_nombre' => $fields['modelo_nombre']['stringValue'] ?? '',
                'modelo_descripcion' => $fields['modelo_descripcion']['stringValue'] ?? '',
                'modelo_url' => $fields['modelo_url']['stringValue'] ?? '',
                'usuario_nombre' => $fields['usuario_nombre']['stringValue'] ?? '',
                'usuario_curso' => $fields['usuario_curso']['stringValue'] ?? '',
                'usuario_email' => $fields['usuario_email']['stringValue'] ?? '',
                'timestamp' => $fields['timestamp']['integerValue'] ?? '',
            ];
        });
    
        // Aplicar filtros
        return $progresos->filter(function ($progreso) use ($search, $startDate, $endDate) {
            $fecha = $progreso['timestamp'] / 1000;
    
            // Filtro por búsqueda general
            $matchesSearch = !$search || stripos($progreso['usuario_nombre'], $search) !== false
                || stripos($progreso['modelo_nombre'], $search) !== false;
    
            // Filtro por fecha
            $matchesDate = (!$startDate || $fecha >= strtotime($startDate))
                && (!$endDate || $fecha <= strtotime($endDate));
    
            return $matchesSearch && $matchesDate;
        });
    }
}    